#include "nrf_drv_rtc.h"
#include "nrf_drv_clock.h"
#include "nrf_gpio.h"
#include "drv_calendar.h"

typedef struct{
  uint8_t msec_tick_cnt;
  uint8_t sec_tick_cnt;
  uint8_t min_tick_cnt;
  uint8_t hour_tick_cnt;
  uint8_t day_tick_cnt;
  uint8_t month_tick_cnt;
  uint8_t year_tick_cnt;
}rtc_timer_t;

#define COMPARE_COUNTERTIME  (3UL)                  //< Get Compare event COMPARE_TIME seconds after the counter starts from 0.

const nrf_drv_rtc_t rtc2 = NRF_DRV_RTC_INSTANCE(2); /**< Declaring an instance of nrf_drv_rtc for RTC2. */

rtc_timer_t rtc_timer;

uint8_t day_of_month[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
//uint8_t leap_year[] = {24, 28, 32, 36, 40, 44, 48, 52, 56, 60, 64, 68, 72, 76, 80};

void sync_rtc(uint8_t yrs, uint8_t mon, uint8_t days, uint8_t hrs, uint8_t mins, uint8_t secs)
{
  rtc_timer.year_tick_cnt       = yrs;
  rtc_timer.month_tick_cnt      = mon;
  rtc_timer.day_tick_cnt        = days;
  rtc_timer.hour_tick_cnt       = hrs;
  rtc_timer.min_tick_cnt        = mins;
  rtc_timer.sec_tick_cnt        = secs;
}

#pragma optimize=none
static void rtc_handler(nrf_drv_rtc_int_type_t int_type)
{
  switch(int_type)
  {
    //case NRF_DRV_RTC_INT_COMPARE0:
    //  break;
    case NRF_DRV_RTC_INT_TICK:
      rtc_timer.msec_tick_cnt++;
      
      if(rtc_timer.msec_tick_cnt >= 8)
      {
        rtc_timer.msec_tick_cnt = 0;
        rtc_timer.sec_tick_cnt++;
        //nrf_gpio_pin_toggle(14);
        if(rtc_timer.sec_tick_cnt >= 60)
        {
          rtc_timer.sec_tick_cnt = 0;
          rtc_timer.min_tick_cnt++;
          
          if(rtc_timer.min_tick_cnt >= 60)
          {
            rtc_timer.min_tick_cnt = 0;
            rtc_timer.hour_tick_cnt++;
            
            if(rtc_timer.hour_tick_cnt >= 24)
            {
              rtc_timer.hour_tick_cnt = 0;
              rtc_timer.day_tick_cnt++;
              
              if(rtc_timer.year_tick_cnt%4 == 0 && rtc_timer.month_tick_cnt == 2)
                day_of_month[1] = 29;
              else
                day_of_month[1] = 28;
                  
              if(rtc_timer.day_tick_cnt > day_of_month[rtc_timer.month_tick_cnt - 1])
              {
                if(rtc_timer.month_tick_cnt == 12)
                {
                  rtc_timer.month_tick_cnt = 0;
                  rtc_timer.year_tick_cnt++;
                }
                rtc_timer.day_tick_cnt = 1;
                rtc_timer.month_tick_cnt++;
              }
            }
          }
        }
      }
      break;
  }
}
#pragma optimize=high
void lfclk_config(void)
{
    ret_code_t err_code = nrf_drv_clock_init();
    APP_ERROR_CHECK(err_code);

    nrf_drv_clock_lfclk_request(NULL);
}

void rtc_config(void)
{
  uint32_t err_code;
  
  //Initialize RTC instance
  nrf_drv_rtc_config_t config = NRF_DRV_RTC_DEFAULT_CONFIG;
  config.prescaler = 4095;
  err_code = nrf_drv_rtc_init(&rtc2, &config, rtc_handler);
  APP_ERROR_CHECK(err_code);
  
  //Enable tick event & interrupt
  nrf_drv_rtc_tick_enable(&rtc2,true);
  
  //Set compare channel to trigger interrupt after COMPARE_COUNTERTIME seconds
  //err_code = nrf_drv_rtc_cc_set(&rtc2,0,COMPARE_COUNTERTIME * 8,true);
  //APP_ERROR_CHECK(err_code);
  
  //Power on RTC instance
  nrf_drv_rtc_enable(&rtc2);
}